def get_schema():
    return dict(
        type='object',
        additionalProperties=False,
        description='Account Service integration',
        required=['extra_attributes', 'credentials'],
        properties=dict(
            extra_attributes=dict(
                type='object',
                additionalProperties=False,
                properties=dict()
            ),
            credentials=dict(
                type='object',
                additionalProperties=False,
                required=[],
                properties=dict()
            )
        )
    )


def connect(extra_attributes, credentials):
    return dict()



def get_schema():
    return dict(
        type='object',
        additionalProperties=False,
        description='BigQuery integration',
        required=['extra_attributes', 'credentials'],
        properties=dict(
            extra_attributes=dict(
                type='object',
                additionalProperties=False,
                properties=dict(
                    project_id=dict(
                        type='string',
                    )
                )
            ),
            credentials=dict(
                type='object',
                additionalProperties=False,
                required=['service_account'],
                properties=dict(
                    service_account=dict(
                        type='object',
                        additionalProperties=False,
                        required=['project_id', 'private_key', 'private_key_id', 'client_email', 'client_id',
                                  'auth_uri',
                                  'token_uri', 'auth_provider_x509_cert_url', 'client_x509_cert_url',
                                  'universe_domain'],
                        properties=dict(
                            project_id=dict(
                                type='string',
                            ),
                            private_key=dict(
                                type='string',
                            ),
                            private_key_id=dict(
                                type='string',
                            ),
                            client_email=dict(
                                type='string',
                            ),
                            client_id=dict(
                                type='string',
                            ),
                            auth_uri=dict(
                                type='string',
                            ),
                            token_uri=dict(
                                type='string',
                            ),
                            auth_provider_x509_cert_url=dict(
                                type='string',
                            ),
                            client_x509_cert_url=dict(
                                type='string',
                            ),
                            universe_domain=dict(
                                type='string',
                            ),
                        )
                    ),
                )
            )

        )
    )


def connect(extra_attributes, credentials):
    from google.cloud import bigquery
    from google.oauth2 import service_account
    credentials = service_account.Credentials.from_service_account_info(credentials['service_account'])
    client = bigquery.Client(credentials=credentials)
    return dict(project_id=client.project)



from jsonpath_ng.ext import parse as ng_parse


def get_schema():
    return dict(
        type='object',
        additionalProperties=False,
        description='Insert Row into BigQuery Table',
        required=['inputs', 'outputs'],
        properties=dict(
            inputs=dict(
                type='object',
                additionalProperties=False,
                required=['project_id', 'dataset_id', 'table_id', 'columns'],
                properties=dict(
                    project_id=dict(
                        type='object',
                        additionalProperties=False,
                        properties=dict(
                            oneOf=[
                                dict(
                                    path=dict(
                                        type='string',
                                        minLength=1,
                                    ),
                                    default=dict(
                                        type='string',
                                    )
                                ),
                                dict(
                                    value='string',
                                    minLength=1,
                                )
                            ]
                        ),
                    ),
                    dataset_id=dict(
                        type='object',
                        additionalProperties=False,
                        properties=dict(
                            oneOf=[
                                dict(
                                    path=dict(
                                        type='string',
                                        minLength=1,
                                    ),
                                    default=dict(
                                        type='string',
                                    )
                                ),
                                dict(
                                    value='string',
                                    minLength=1,
                                )
                            ]
                        ),
                    ),
                    table_id=dict(
                        type='object',
                        additionalProperties=False,
                        properties=dict(
                            oneOf=[
                                dict(
                                    path=dict(
                                        type='string',
                                        minLength=1,
                                    ),
                                    default=dict(
                                        type='string',
                                    )
                                ),
                                dict(
                                    value='string',
                                    minLength=1,
                                )
                            ]
                        ),
                    ),
                    records=dict(
                        type='object',
                        additionalProperties=False,
                        properties=dict(
                            path=dict(
                                type='string',
                            )
                        )
                    ),
                    table_schema=dict(
                        type='array',
                        minItems=1,
                        items=dict(
                            type='object',
                            additionalProperties=False,
                            required=['name', 'type'],
                            properties=dict(
                                name=dict(
                                    type='string',
                                ),
                                type=dict(
                                    type='string',
                                    enum=['STRING', 'INTEGER', 'NUMERIC', 'BOOLEAN', 'TIMESTAMP']
                                ),
                                path=dict(
                                    type='string',
                                ),
                                value=dict(
                                    type=['string', 'integer', 'number', 'boolean', 'null'],
                                ),
                                default=dict(
                                    type=['string', 'integer', 'number', 'boolean', 'null'],
                                )
                            )
                        )
                    )
                )
            ),
            outputs=dict(
                type='object',
                additionalProperties=False,
                required=['row_id'],
                properties=dict(
                    row_id=dict(
                        type='string',
                    ),
                    inserted_ts=dict(
                        type='integer',
                    )
                )
            ),
        )
    )


def process(workflow_uuid, account_uuid, node_uuid, trigger_uuid, ingested_ts, processed_ts, inputs, integration,
            **kwargs):
    from google.cloud import bigquery
    from google.oauth2 import service_account
    credentials = service_account.Credentials.from_service_account_info(integration['credentials']['service_account'])
    client = bigquery.Client(credentials=credentials)
    table = client.get_table(
        table='{project_id}.{dataset_id}.{table_id}'.format(
            project_id=inputs['project_id'],
            dataset_id=inputs['dataset_id'],
            table_id=inputs['table_id'],
        )
    )
    client.insert_rows_json(
        table=table,
        json_rows=format_rows(data=inputs['records'], workflow_uuid=workflow_uuid, account_uuid=account_uuid,
                              node_uuid=node_uuid, trigger_uuid=trigger_uuid, ingested_ts=ingested_ts,
                              processed_ts=processed_ts, schema=inputs['table_schema'])
    )
    return dict(
        job_id='test_job_id',
        slot_time_ms=100,
        file_pattern='test_file_pattern',
    )


def format_rows(data, workflow_uuid, account_uuid, node_uuid, trigger_uuid, ingested_ts, processed_ts, schema):
    if not isinstance(data, list):
        data = [data]
    rows = list()
    for record in data:
        row = dict(
            account_uuid=account_uuid,
            workflow_uuid=workflow_uuid,
            node_uuid=node_uuid,
            trigger_uuid=trigger_uuid,
            ingested_ts=ingested_ts,
            processed_ts=processed_ts,
        )
        for column in schema:
            value = None
            if 'path' in column:
                parser = ng_parse(column['path'])
                value = [i.value for i in parser.find(record)]
                if len(value) == 0:
                    value = None
                elif len(value) == 1:
                    value = value[0]
            elif 'value' in column:
                value = column['value']
            if value is None and 'default' in column:
                value = column['default']
            row[column['name']] = value
        rows.append(
            record
        )
    return rows



#!/usr/bin/env python

"""The setup script."""
import os

from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'google-cloud-bigquery',
    'requests',
    'openai',
    'jsonpath-ng',
]

test_requirements = [
    'pytest>=3',
    'responses'

]

setup(
    author="Perry Stallings",
    author_email='astal01@gmail.com',
    python_requires='>=3.8',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Python Integration Project Framework",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='parrot_integrations_core',
    name='parrot-integrations-core',
    packages=[f'parrot_integrations.{i}' for i in os.listdir('./parrot_integrations') if os.path.isdir(i)],
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/perrystallings/parrot_integrations_core',
    version='0.0.5',
    zip_safe=False
)



def get_schema() -> dict:
    return dict(
        type='object',
        additionalProperties=False,
        description='Execute BQ Query',
        required=['inputs', 'outputs'],
        properties=dict(
            expand_results=dict(
                type='boolean',
                default=True,
            ),
            inputs=dict(
                type='object',
                additionalProperties=False,
                required=['project_id', 'query', 'output_to_file'],
                properties=dict(
                    project_id=dict(
                        type='string',
                    ),
                    query=dict(
                        type='string',
                    ),
                    output_to_file=dict(
                        type='boolean',
                        default=True,
                    ),
                    file_pattern=dict(
                        type='string',
                    ),
                )
            ),
            outputs=dict(
                type='object',
                additionalProperties=False,
                required=['job_id', 'slot_time_ms'],
                properties=dict(
                    job_id=dict(
                        type='string',
                    ),
                    slot_time_ms=dict(
                        type='integer',
                    ),
                    files=dict(
                        type='array',
                        items=dict(
                            type='string',
                        )
                    ),
                    file=dict(
                        type='string'
                    ),
                    rows=dict(
                        type='array',
                        items=dict(
                            type='object'
                        )
                    ),
                    row=dict(
                        type='object',
                    )
                )
            )
        )
    )


def process(node_uuid, workflow_uuid, inputs, expand_results, integration):
    return dict(
        job_id='test_job_id',
        slot_time_ms=100,
        file_pattern='test_file_pattern',
    )
